@@ -34,10 +34,12 @@
 from json import load, dumps
 from os import path
 from logging import getLogger
+from io import StringIO
+from pydoc import pager
 
 from kamaki.cli import command
 from kamaki.cli.command_tree import CommandTree
-from kamaki.cli.utils import print_dict, print_json, filter_dicts_by_dict
+from kamaki.cli.utils import filter_dicts_by_dict
 from kamaki.clients.image import ImageClient
 from kamaki.clients.pithos import PithosClient
 from kamaki.clients.astakos import AstakosClient
@@ -81,9 +83,8 @@ class _init_image(_command_init):
         if getattr(self, 'cloud', None):
             img_url = self._custom_url('image') or self._custom_url('plankton')
             if img_url:
-                token = self._custom_token('image')\
-                    or self._custom_token('plankton')\
-                    or self.config.get_cloud(self.cloud, 'token')
+                token = self._custom_token('image') or self._custom_token(
+                    'plankton') or self.config.get_cloud(self.cloud, 'token')
                 self.client = ImageClient(base_url=img_url, token=token)
                 return
         if getattr(self, 'auth_base', False):
@@ -260,18 +261,22 @@ class image_list(_init_image, _optional_json, _name_filter, _id_filter):
         images = self._filter_by_id(images)
         images = self._non_exact_name_filter(images)
 
-        if self['detail'] and not self['json_output']:
+        if self['detail'] and not (
+                self['json_output'] or self['output_format']):
             images = self._add_owner_name(images)
         elif detail and not self['detail']:
             for img in images:
                 for key in set(img).difference(self.PERMANENTS):
                     img.pop(key)
         kwargs = dict(with_enumeration=self['enum'])
-        if self['more']:
-            kwargs['page_size'] = self['limit'] or 10
-        elif self['limit']:
+        if self['limit']:
             images = images[:self['limit']]
+        if self['more']:
+            kwargs['out'] = StringIO()
+            kwargs['title'] = ()
         self._print(images, **kwargs)
+        if self['more']:
+            pager(kwargs['out'].getvalue())
 
     def main(self):
         super(self.__class__, self)._run()
@@ -297,9 +302,9 @@ class image_info(_init_image, _optional_json):
     @errors.plankton.id
     def _run(self, image_id):
         meta = self.client.get_meta(image_id)
-        if not self['json_output']:
+        if not (self['json_output'] or self['output_format']):
             meta['owner'] += ' (%s)' % self._uuid2username(meta['owner'])
-        self._print(meta, print_dict)
+        self._print(meta, self.print_dict)
 
     def main(self, image_id):
         super(self.__class__, self)._run()
@@ -373,9 +378,8 @@ class image_meta_delete(_init_image, _optional_output_cmd):
     )
 
     def _check_empty(self):
-        for term in (
-                'disk_format', 'container_format', 'status', 'properties'):
-            if self[term]:
+        for t in ('disk_format', 'container_format', 'status', 'properties'):
+            if self[t]:
                 return
         raiseCLIError(
             'Nothing to update, please use arguments (-h for a list)')
@@ -404,11 +408,11 @@ class image_meta_delete(_init_image, _optional_output_cmd):
 class image_register(_init_image, _optional_json):
     """(Re)Register an image file to an Image service
     The image file must be stored at a pithos repository
-    Some metadata can be set by user (e.g. disk-format) while others are set
-    only automatically (e.g. image id). There are also some custom user
+    Some metadata can be set by user (e.g., disk-format) while others are set
+    only automatically (e.g., image id). There are also some custom user
     metadata, called properties.
     A register command creates a remote meta file at
-       <container>:<image path>.meta
+    .  <container>:<image path>.meta
     Users may download and edit this file and use it to re-register one or more
     images.
     In case of a meta file, runtime arguments for metadata or properties
@@ -420,8 +424,7 @@ class image_register(_init_image, _optional_json):
     arguments = dict(
         checksum=ValueArgument('Set image checksum', '--checksum'),
         container_format=ValueArgument(
-            'Set container format',
-            '--container-format'),
+            'Set container format', '--container-format'),
         disk_format=ValueArgument('Set disk format', '--disk-format'),
         owner_name=ValueArgument('Set user uuid by user name', '--owner-name'),
         properties=KeyValueArgument(
@@ -456,8 +459,8 @@ class image_register(_init_image, _optional_json):
         if getattr(self, 'auth_base', False):
             return self.auth_base.term('id', atoken)
         else:
-            astakos_url = self.config.get('user', 'url')\
-                or self.config.get('astakos', 'url')
+            astakos_url = self.config.get('user', 'url') or self.config.get(
+                'astakos', 'url')
             if not astakos_url:
                 raise CLIBaseUrlError(service='astakos')
             user = AstakosClient(astakos_url, atoken)
@@ -489,7 +492,7 @@ class image_register(_init_image, _optional_json):
             try:
                 for k, v in _load_image_meta(pfile).items():
                     key = k.lower().replace('-', '_')
-                    if k == 'properties':
+                    if key == 'properties':
                         for pk, pv in v.items():
                             properties[pk.upper().replace('-', '_')] = pv
                     elif key == 'name':
@@ -625,7 +628,7 @@ class image_register(_init_image, _optional_json):
                             img_path, dst_cont)] + howto_image_file)
             raise
         r['owner'] += ' (%s)' % self._uuid2username(r['owner'])
-        self._print(r, print_dict)
+        self._print(r, self.print_dict)
 
         #upload the metadata file
         if pclient:
@@ -634,14 +637,15 @@ class image_register(_init_image, _optional_json):
                     meta_path, dumps(r, indent=2),
                     container_info_cache=self.container_info_cache)
             except TypeError:
-                print('Failed to dump metafile %s:%s' % (dst_cont, meta_path))
+                self.error(
+                    'Failed to dump metafile %s:%s' % (dst_cont, meta_path))
                 return
-            if self['json_output']:
-                print_json(dict(
+            if self['json_output'] or self['output_format']:
+                self.print_json(dict(
                     metafile_location='%s:%s' % (dst_cont, meta_path),
                     headers=meta_headers))
             else:
-                print('Metadata file uploaded as %s:%s (version %s)' % (
+                self.error('Metadata file uploaded as %s:%s (version %s)' % (
                     dst_cont, meta_path, meta_headers['x-object-version']))
 
     def main(self, name, container___image_path):
@@ -696,7 +700,7 @@ class image_members_list(_init_image, _optional_json):
     @errors.plankton.id
     def _run(self, image_id):
         members = self.client.list_members(image_id)
-        if not self['json_output']:
+        if not (self['json_output'] or self['output_format']):
             uuids = [member['member_id'] for member in members]
             usernames = self._uuids2usernames(uuids)
             for member in members:
@@ -770,9 +774,7 @@ class image_compute_list(
     arguments = dict(
         detail=FlagArgument('show detailed output', ('-l', '--details')),
         limit=IntArgument('limit number listed images', ('-n', '--number')),
-        more=FlagArgument(
-            'output results in pages (-n to set items per page, default 10)',
-            '--more'),
+        more=FlagArgument('handle long lists of results', '--more'),
         enum=FlagArgument('Enumerate results', '--enumerate'),
         user_id=ValueArgument('filter by user_id', '--user-id'),
         user_name=ValueArgument('filter by username', '--user-name'),
@@ -820,18 +822,22 @@ class image_compute_list(
             images = self._filter_by_user(images)
         if withmeta:
             images = self._filter_by_metadata(images)
-        if self['detail'] and not self['json_output']:
+        if self['detail'] and not (
+                self['json_output'] or self['output_format']):
             images = self._add_name(self._add_name(images, 'tenant_id'))
         elif detail and not self['detail']:
             for img in images:
                 for key in set(img).difference(self.PERMANENTS):
                     img.pop(key)
         kwargs = dict(with_enumeration=self['enum'])
-        if self['more']:
-            kwargs['page_size'] = self['limit'] or 10
-        elif self['limit']:
+        if self['limit']:
             images = images[:self['limit']]
+        if self['more']:
+            kwargs['out'] = StringIO()
+            kwargs['title'] = ()
         self._print(images, **kwargs)
+        if self['more']:
+            pager(kwargs['out'].getvalue())
 
     def main(self):
         super(self.__class__, self)._run()
@@ -851,7 +857,7 @@ class image_compute_info(_init_cyclades, _optional_json):
         usernames = self._uuids2usernames(uuids)
         image['user_id'] += ' (%s)' % usernames[image['user_id']]
         image['tenant_id'] += ' (%s)' % usernames[image['tenant_id']]
-        self._print(image, print_dict)
+        self._print(image, self.print_dict)
 
     def main(self, image_id):
         super(self.__class__, self)._run()
@@ -886,7 +892,7 @@ class image_compute_properties_list(_init_cyclades, _optional_json):
     @errors.cyclades.connection
     @errors.plankton.id
     def _run(self, image_id):
-        self._print(self.client.get_image_metadata(image_id), print_dict)
+        self._print(self.client.get_image_metadata(image_id), self.print_dict)
 
     def main(self, image_id):
         super(self.__class__, self)._run()
@@ -902,30 +908,14 @@ class image_compute_properties_get(_init_cyclades, _optional_json):
     @errors.plankton.id
     @errors.plankton.metadata
     def _run(self, image_id, key):
-        self._print(self.client.get_image_metadata(image_id, key), print_dict)
+        self._print(
+            self.client.get_image_metadata(image_id, key), self.print_dict)
 
     def main(self, image_id, key):
         super(self.__class__, self)._run()
         self._run(image_id=image_id, key=key)
 
 
-#@command(image_cmds)
-#class image_compute_properties_add(_init_cyclades, _optional_json):
-#    """Add a property to an image"""
-#
-#    @errors.generic.all
-#    @errors.cyclades.connection
-#    @errors.plankton.id
-#    @errors.plankton.metadata
-#    def _run(self, image_id, key, val):
-#        self._print(
-#            self.client.create_image_metadata(image_id, key, val), print_dict)
-#
-#    def main(self, image_id, key, val):
-#        super(self.__class__, self)._run()
-#        self._run(image_id=image_id, key=key, val=val)
-
-
 @command(image_cmds)
 class image_compute_properties_set(_init_cyclades, _optional_json):
     """Add / update a set of properties for an image
@@ -942,7 +932,8 @@ class image_compute_properties_set(_init_cyclades, _optional_json):
             key, sep, val = keyval.partition('=')
             meta[key] = val
         self._print(
-            self.client.update_image_metadata(image_id, **meta), print_dict)
+            self.client.update_image_metadata(image_id, **meta),
+            self.print_dict)
 
     def main(self, image_id, *key_equals_value):
         super(self.__class__, self)._run()
